@@ -4,7 +4,7 @@ use warnings;
 package OPCUA::Open62541::Test::Client;
 use OPCUA::Open62541::Test::Logger;
 use OPCUA::Open62541 qw(STATUSCODE_GOOD STATUSCODE_BADCONNECTIONCLOSED
-    :CLIENTSTATE);
+    :CLIENTSTATE :SESSIONSTATE);
 use Carp 'croak';
 use Time::HiRes qw(sleep);
 
@@ -18,11 +18,15 @@ sub planning {
 sub new {
     my $class = shift;
     my $self = { @_ };
-    $self->{port}
-	or croak "no port given";
     $self->{timeout} ||= 10;
     $self->{logfile} //= "client.log";
 
+    if (not $self->{url}) {
+	$self->{host} ||= "localhost";
+	$self->{url} = "opc.tcp://$self->{host}";
+	$self->{url} .= ":$self->{port}" if $self->{port};
+    }
+
     ok($self->{client} = OPCUA::Open62541::Client->new(), "client: new");
     ok($self->{config} = $self->{client}->getConfig(), "client: get config");
 
@@ -39,9 +43,6 @@ sub start {
     my OPCUA::Open62541::Test::Client $self = shift;
 
     is($self->{config}->setDefault(), "Good", "client: set default config");
-    $self->{url} = "opc.tcp://localhost";
-    $self->{url} .= ":$self->{port}" if $self->{port};
-
     ok($self->{logger} = $self->{config}->getLogger(), "client: get logger");
     ok($self->{log} = OPCUA::Open62541::Test::Logger->new(
 	logger => $self->{logger},
@@ -58,10 +59,17 @@ sub run {
     note("going to connect client to url $self->{url}");
     is($self->{client}->connect($self->{url}), STATUSCODE_GOOD,
 	"client: connect");
-    is($self->{client}->getState(), CLIENTSTATE_SESSION,
-	"client: state session");
+    if (defined &CLIENTSTATE_SESSION) {
+	is($self->{client}->getState(), CLIENTSTATE_SESSION,
+	    "client: state session");
+    } else {
+	my ($channel, $session, $connect) = $self->{client}->getState();
+	is($session, SESSIONSTATE_ACTIVATED,
+	    "client: state session activated");
+    }
     # check client did connect(2)
-    ok($self->{log}->loggrep(qr/TCP connection established/, 5),
+    ok($self->{log}->loggrep(
+	qr/TCP connection established|SessionState: Activated/, 5),
 	"client: log grep connected");
 
     return $self;
@@ -105,8 +113,14 @@ sub stop {
 
     note("going to disconnect client");
     is($self->{client}->disconnect(), STATUSCODE_GOOD, "client: disconnect");
-    is($self->{client}->getState, CLIENTSTATE_DISCONNECTED,
-	"client: state disconnected");
+    if (defined &CLIENTSTATE_DISCONNECTED) {
+	is($self->{client}->getState(), CLIENTSTATE_DISCONNECTED,
+	    "client: state disconnected");
+    } else {
+	my ($channel, $session, $connect) = $self->{client}->getState();
+	is($session, SESSIONSTATE_CLOSED,
+	    "client: state session closing");
+    }
 
     return $self;
 }
@@ -155,9 +169,17 @@ Create a new test client instance.
 
 =over 8
 
+=item $args{host}
+
+Hostname or IP of the server. Defaults to localhost.
+
 =item $args{port}
 
-Required port number of the server.
+Optional port number of the server.
+
+=item $args{url}
+
+URL of the server. Overwrites host and port arguments.
 
 =item $args{logfile}
 
@@ -173,9 +195,8 @@ Defaults to 10 seconds.
 
 =item $client->url($url)
 
-Optionally set the url.
-Returns the url created from localhost and port.
-Must be called after start() for that.
+Returns the URL of the server.
+Can also set the URL by passing an argument.
 
 =item $client->start()
 
@@ -221,7 +242,7 @@ Alexander Bluhm E<lt>bluhm@genua.deE<gt>,
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2020 Alexander Bluhm
+Copyright (c) 2020-2022 Alexander Bluhm
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
